@@ -1,9 +1,10 @@
-"""Page source management and filtering service."""
+"""Page source management and filtering service for web and mobile."""
 
 import logging
-from typing import Any, Dict, List, Optional
+import xml.etree.ElementTree as ET
+from typing import Any, Dict, List, Optional, Union
 
-from robotmcp.models.session_models import ExecutionSession
+from robotmcp.models.session_models import ExecutionSession, PlatformType
 from robotmcp.models.config_models import ExecutionConfig
 
 logger = logging.getLogger(__name__)
@@ -19,7 +20,7 @@ except ImportError:
 
 
 class PageSourceService:
-    """Manages page source retrieval, filtering, and context extraction."""
+    """Manages page source retrieval, filtering, and context extraction for web and mobile."""
     
     def __init__(self, config: Optional[ExecutionConfig] = None):
         self.config = config or ExecutionConfig()
@@ -166,7 +167,7 @@ class PageSourceService:
         filtering_level: str = "standard"
     ) -> Dict[str, Any]:
         """
-        Get page source for a session.
+        Get page source for a session (web or mobile).
         
         Args:
             session: ExecutionSession to get page source from
@@ -176,23 +177,76 @@ class PageSourceService:
             filtering_level: Filtering intensity when filtered=True ('minimal', 'standard', 'aggressive').
         """
         try:
-            # Try to get fresh page source using browser library
+            # Prefer web path if a web automation library is active/selected
+            active_lib = None
+            try:
+                active_lib = session.get_active_library()
+            except Exception:
+                active_lib = None
+            web_pref = None
+            try:
+                web_pref = session.get_web_automation_library()
+            except Exception:
+                web_pref = None
+
+            if (
+                (active_lib in ("browser", "selenium"))
+                or (web_pref in ("Browser", "SeleniumLibrary"))
+            ):
+                # Continue with web page source retrieval below
+                pass
+            elif session.is_mobile_session():
+                # Only use mobile source path when not in a web automation session
+                return await self._get_mobile_source(
+                    session, full_source, filtered, filtering_level
+                )
+            
+            # RF-context only: get page source directly through the session RF context
             page_source = ""
             try:
-                page_source = await self._get_page_source_unified_async(session, browser_library_manager)
-                if page_source:
-                    session.browser_state.page_source = page_source
-                    # Also update URL and title if we can extract them
-                    url = await self._get_current_url(session, browser_library_manager)
-                    if url and url != "about:blank":
-                        session.browser_state.current_url = url
-                    title = await self._get_page_title(session, browser_library_manager)
-                    if title and title != "Generic Page":
-                        session.browser_state.page_title = title
-                else:
-                    page_source = session.browser_state.page_source or ""
+                from robotmcp.components.execution.rf_native_context_manager import (
+                    get_rf_native_context_manager,
+                )
+
+                rf_mgr = get_rf_native_context_manager()
+                # Try Browser first, then Selenium keyword name
+                for kw in ("Get Page Source", "Get Source"):
+                    res = rf_mgr.execute_keyword_with_context(
+                        session_id=session.session_id,
+                        keyword_name=kw,
+                        arguments=[],
+                        assign_to=None,
+                        session_variables=dict(session.variables),
+                    )
+                    if res and res.get("success"):
+                        out = res.get("output") or res.get("result")
+                        if isinstance(out, str) and out:
+                            page_source = out
+                            session.browser_state.page_source = out
+                            break
+                # Update URL and title, best-effort
+                try:
+                    res = rf_mgr.execute_keyword_with_context(
+                        session_id=session.session_id,
+                        keyword_name="Get Url",
+                        arguments=[],
+                    )
+                    if res and res.get("success") and res.get("output"):
+                        session.browser_state.current_url = res.get("output")
+                except Exception:
+                    pass
+                try:
+                    res = rf_mgr.execute_keyword_with_context(
+                        session_id=session.session_id,
+                        keyword_name="Get Title",
+                        arguments=[],
+                    )
+                    if res and res.get("success") and res.get("output"):
+                        session.browser_state.page_title = res.get("output")
+                except Exception:
+                    pass
             except Exception as e:
-                logger.debug(f"Could not get fresh page source: {e}")
+                logger.debug(f"RF-context page source retrieval failed: {e}")
                 page_source = session.browser_state.page_source or ""
             
             if not page_source:
@@ -251,6 +305,8 @@ class PageSourceService:
                         )
                     else:
                         result["page_source_preview"] = page_source
+                    # Add generic 'source' field for compatibility with older tests
+                    result["source"] = result["page_source_preview"]
             
             return result
             
@@ -273,92 +329,68 @@ class PageSourceService:
             str: Page source HTML or None if not available
         """
         try:
-            from robotmcp.core.dynamic_keyword_orchestrator import get_keyword_discovery
-            
-            # Use the same keyword discovery system that handles Browser Library execution
-            keyword_discovery = get_keyword_discovery()
-            
-            # Determine which library to use
-            library, library_type = browser_library_manager.get_active_browser_library(session)
-            
-            if library_type == "browser":
-                # Use Browser Library via keyword discovery
-                result = await keyword_discovery.execute_keyword("Get Page Source", [], session.variables)
-                if result and result.get("success") and result.get("output"):
-                    return result["output"]
-                    
-            elif library_type == "selenium":
-                # Use SeleniumLibrary via keyword discovery  
-                result = await keyword_discovery.execute_keyword("Get Source", [], session.variables)
-                if result and result.get("success") and result.get("output"):
-                    return result["output"]
-                    
-            else:
-                logger.debug("No active browser library available for page source")
-                return None
-                
-        except Exception as e:
-            logger.error(f"Error getting page source via keyword discovery: {e}")
-            
-            # Fallback to direct library calls if keyword discovery fails
-            try:
-                library, library_type = browser_library_manager.get_active_browser_library(session)
-                
-                if library_type == "browser" and library:
-                    # Browser Library - direct call as fallback
-                    return library.get_page_source()
-                    
-                elif library_type == "selenium" and library:
-                    # SeleniumLibrary - direct call as fallback
-                    return library.get_source()
-                    
-            except Exception as fallback_error:
-                logger.error(f"Fallback page source retrieval also failed: {fallback_error}")
-                
-            return None
+            from robotmcp.components.execution.rf_native_context_manager import (
+                get_rf_native_context_manager,
+            )
+
+            mgr = get_rf_native_context_manager()
+            # Try Browser first, then Selenium variants using the RF session
+            for kw in ("Get Page Source", "Get Source"):
+                res = mgr.execute_keyword_with_context(
+                    session_id=session.session_id,
+                    keyword_name=kw,
+                    arguments=[],
+                    assign_to=None,
+                    session_variables=dict(session.variables),
+                )
+                if res and res.get("success"):
+                    out = res.get("output") or res.get("result")
+                    if isinstance(out, str) and out:
+                        # Cache for later
+                        session.browser_state.page_source = out
+                        return out
+        except Exception as rf_fallback_err:
+            logger.debug(f"RF-context page source retrieval not available: {rf_fallback_err}")
+
+        return None
     
     async def _get_current_url(self, session: ExecutionSession, browser_library_manager: Any) -> Optional[str]:
-        """Get current URL using keyword discovery."""
+        """Get current URL using RF context in the session."""
         try:
-            from robotmcp.core.dynamic_keyword_orchestrator import get_keyword_discovery
-            
-            keyword_discovery = get_keyword_discovery()
-            library, library_type = browser_library_manager.get_active_browser_library(session)
-            
-            if library_type == "browser":
-                result = await keyword_discovery.execute_keyword("Get Url", [], session.variables)
-                if result and result.get("success") and result.get("output"):
-                    return result["output"]
-            elif library_type == "selenium":
-                result = await keyword_discovery.execute_keyword("Get Location", [], session.variables)
-                if result and result.get("success") and result.get("output"):
-                    return result["output"]
-                    
+            from robotmcp.components.execution.rf_native_context_manager import (
+                get_rf_native_context_manager,
+            )
+
+            mgr = get_rf_native_context_manager()
+            for kw in ("Get Url", "Get Location"):
+                res = mgr.execute_keyword_with_context(
+                    session_id=session.session_id,
+                    keyword_name=kw,
+                    arguments=[],
+                )
+                if res and res.get("success") and res.get("output"):
+                    return res["output"]
         except Exception as e:
             logger.debug(f"Could not get current URL: {e}")
-            
         return None
     
     async def _get_page_title(self, session: ExecutionSession, browser_library_manager: Any) -> Optional[str]:
-        """Get page title using keyword discovery."""
+        """Get page title using RF context in the session."""
         try:
-            from robotmcp.core.dynamic_keyword_orchestrator import get_keyword_discovery
-            
-            keyword_discovery = get_keyword_discovery()
-            library, library_type = browser_library_manager.get_active_browser_library(session)
-            
-            if library_type == "browser":
-                result = await keyword_discovery.execute_keyword("Get Title", [], session.variables)
-                if result and result.get("success") and result.get("output"):
-                    return result["output"]
-            elif library_type == "selenium":
-                result = await keyword_discovery.execute_keyword("Get Title", [], session.variables)  
-                if result and result.get("success") and result.get("output"):
-                    return result["output"]
-                    
+            from robotmcp.components.execution.rf_native_context_manager import (
+                get_rf_native_context_manager,
+            )
+
+            mgr = get_rf_native_context_manager()
+            res = mgr.execute_keyword_with_context(
+                session_id=session.session_id,
+                keyword_name="Get Title",
+                arguments=[],
+            )
+            if res and res.get("success") and res.get("output"):
+                return res["output"]
         except Exception as e:
             logger.debug(f"Could not get page title: {e}")
-            
         return None
 
     async def extract_page_context(self, html: str) -> Dict[str, Any]:
@@ -528,4 +560,304 @@ class PageSourceService:
         Returns:
             list: List of supported filtering level names
         """
-        return ["minimal", "standard", "aggressive"]
\ No newline at end of file
+        return ["minimal", "standard", "aggressive"]
+    
+    async def _get_mobile_source(
+        self,
+        session: ExecutionSession,
+        full_source: bool = False,
+        filtered: bool = False,
+        filtering_level: str = "standard"
+    ) -> Dict[str, Any]:
+        """
+        Get source from mobile app using AppiumLibrary.
+        
+        Args:
+            session: Mobile session
+            full_source: Return complete source
+            filtered: Apply filtering
+            filtering_level: Filtering intensity
+            
+        Returns:
+            Mobile app source and metadata
+        """
+        try:
+            # Check if AppiumLibrary is loaded
+            if 'AppiumLibrary' not in session.imported_libraries:
+                return {
+                    "success": False,
+                    "error": "AppiumLibrary not loaded in session"
+                }
+            
+            # Use RF context manager to execute AppiumLibrary keyword in-session
+            try:
+                from robotmcp.components.execution.rf_native_context_manager import (
+                    get_rf_native_context_manager,
+                )
+                rf_mgr = get_rf_native_context_manager()
+                res = rf_mgr.execute_keyword_with_context(
+                    session_id=session.session_id,
+                    keyword_name="Get Source",
+                    arguments=[],
+                    assign_to=None,
+                    session_variables=dict(session.variables),
+                )
+                if res and res.get("success"):
+                    if res.get("output"):
+                        source = res["output"]
+                    else:
+                        source = res.get("result")
+                else:
+                    error_msg = res.get("error", "Unknown error") if res else "No result"
+                    logger.debug(f"RF context call failed for Get Source: {error_msg}")
+                    source = await self._get_mobile_source_direct(session)
+                    if not source:
+                        return {
+                            "success": False,
+                            "error": f"Failed to get mobile source: {error_msg}"
+                        }
+            except Exception as rf_err:
+                logger.debug(f"RF context execution failed for Get Source: {rf_err}")
+                source = await self._get_mobile_source_direct(session)
+                if not source:
+                    return {
+                        "success": False,
+                        "error": f"No active mobile app session: {str(rf_err)}"
+                    }
+            
+            if not source:
+                return {
+                    "success": False,
+                    "error": "No source available from mobile app"
+                }
+            
+            # Parse mobile source based on platform
+            platform = session.mobile_config.platform_name if session.mobile_config else None
+            
+            if platform == 'Android':
+                parsed_source = self._parse_android_source(source, filtered)
+            elif platform == 'iOS':
+                parsed_source = self._parse_ios_source(source, filtered)
+            else:
+                parsed_source = {"raw_source": source}
+            
+            result = {
+                "success": True,
+                "session_id": session.session_id,
+                "platform": platform,
+                "source_type": "mobile_app",
+                "source_length": len(source) if isinstance(source, str) else 0,
+                "current_context": session.current_context or "NATIVE_APP"
+            }
+            
+            if session.mobile_config:
+                result["app_info"] = {
+                    "package": session.mobile_config.app_package,
+                    "activity": session.mobile_config.app_activity,
+                    "device": session.mobile_config.device_name
+                }
+            
+            if full_source:
+                result["source"] = source
+                result["parsed_structure"] = parsed_source
+            else:
+                # Return preview
+                preview_size = self.config.PAGE_SOURCE_PREVIEW_SIZE
+                if isinstance(source, str) and len(source) > preview_size:
+                    result["source_preview"] = source[:preview_size] + "...[Truncated]"
+                else:
+                    result["source_preview"] = source
+                result["parsed_structure_summary"] = self._get_structure_summary(parsed_source)
+            
+            return result
+            
+        except Exception as e:
+            logger.error(f"Error getting mobile source: {e}")
+            return {
+                "success": False,
+                "error": f"Failed to get mobile source: {str(e)}"
+            }
+    
+    def _parse_android_source(self, xml_source: str, filtered: bool = False) -> Dict[str, Any]:
+        """Parse Android UI XML hierarchy."""
+        try:
+            root = ET.fromstring(xml_source)
+            
+            def parse_element(elem, depth=0):
+                # Skip certain elements if filtering
+                if filtered and elem.tag in ['hierarchy', 'android.view.ViewGroup']:
+                    if not elem.get('resource-id') and not elem.get('content-desc'):
+                        # Skip container elements without IDs
+                        children = []
+                        for child in elem:
+                            children.extend(parse_element(child, depth))
+                        return children
+                
+                element_info = {
+                    'tag': elem.tag.split('.')[-1] if '.' in elem.tag else elem.tag,
+                    'class': elem.get('class', ''),
+                    'resource_id': elem.get('resource-id', ''),
+                    'text': elem.get('text', ''),
+                    'content_desc': elem.get('content-desc', ''),
+                    'enabled': elem.get('enabled') == 'true',
+                    'clickable': elem.get('clickable') == 'true',
+                    'bounds': elem.get('bounds', ''),
+                    'depth': depth
+                }
+                
+                # Only include non-empty fields
+                element_info = {k: v for k, v in element_info.items() if v or v is False}
+                
+                # Parse children
+                children = []
+                for child in elem:
+                    children.extend(parse_element(child, depth + 1))
+                
+                if children:
+                    element_info['children'] = children
+                    
+                return [element_info]
+            
+            parsed = parse_element(root)
+            return {"elements": parsed, "type": "android_xml"}
+            
+        except Exception as e:
+            logger.error(f"Error parsing Android source: {e}")
+            return {"error": str(e), "raw": xml_source[:500] if len(xml_source) > 500 else xml_source}
+    
+    def _parse_ios_source(self, source: Union[str, dict], filtered: bool = False) -> Dict[str, Any]:
+        """Parse iOS XCUITest hierarchy."""
+        try:
+            if isinstance(source, dict):
+                # Already parsed as dict
+                return {"elements": [source], "type": "ios_dict"}
+            elif isinstance(source, str):
+                # Try to parse as XML
+                root = ET.fromstring(source)
+                
+                def parse_element(elem, depth=0):
+                    element_info = {
+                        'type': elem.get('type', ''),
+                        'name': elem.get('name', ''),
+                        'label': elem.get('label', ''),
+                        'value': elem.get('value', ''),
+                        'enabled': elem.get('enabled') == 'true',
+                        'visible': elem.get('visible') == 'true',
+                        'x': elem.get('x', ''),
+                        'y': elem.get('y', ''),
+                        'width': elem.get('width', ''),
+                        'height': elem.get('height', ''),
+                        'depth': depth
+                    }
+                    
+                    # Only include non-empty fields
+                    element_info = {k: v for k, v in element_info.items() if v or v is False}
+                    
+                    # Parse children
+                    children = []
+                    for child in elem:
+                        children.append(parse_element(child, depth + 1))
+                    
+                    if children:
+                        element_info['children'] = children
+                        
+                    return element_info
+                
+                parsed = parse_element(root)
+                return {"elements": [parsed], "type": "ios_xml"}
+            else:
+                return {"raw": str(source), "type": "unknown"}
+                
+        except Exception as e:
+            logger.error(f"Error parsing iOS source: {e}")
+            return {"error": str(e), "raw": str(source)[:500] if len(str(source)) > 500 else str(source)}
+    
+    def _get_structure_summary(self, parsed_source: Dict[str, Any]) -> Dict[str, Any]:
+        """Get summary of parsed mobile structure."""
+        if 'error' in parsed_source:
+            return parsed_source
+            
+        elements = parsed_source.get('elements', [])
+        if not elements:
+            return {"element_count": 0}
+            
+        # Count different element types
+        def count_elements(elem_list):
+            counts = {}
+            total = 0
+            for elem in elem_list:
+                if isinstance(elem, dict):
+                    elem_type = elem.get('tag') or elem.get('type') or elem.get('class', 'unknown')
+                    counts[elem_type] = counts.get(elem_type, 0) + 1
+                    total += 1
+                    
+                    if 'children' in elem:
+                        child_counts, child_total = count_elements(elem['children'])
+                        for k, v in child_counts.items():
+                            counts[k] = counts.get(k, 0) + v
+                        total += child_total
+                        
+            return counts, total
+        
+        type_counts, total_count = count_elements(elements)
+        
+        return {
+            "total_elements": total_count,
+            "element_types": type_counts,
+            "structure_type": parsed_source.get('type', 'unknown')
+        }
+    
+    async def _get_mobile_source_direct(self, session: ExecutionSession) -> Optional[str]:
+        """
+        Fallback method to get mobile source via direct library method call.
+        
+        This method attempts to get the AppiumLibrary instance directly and call
+        the get_source method without going through Robot Framework's BuiltIn.run_keyword
+        which requires an execution context.
+        
+        Args:
+            session: ExecutionSession containing mobile configuration
+            
+        Returns:
+            str: Mobile app source XML or None if failed
+        """
+        try:
+            # Try to get AppiumLibrary instance from the library manager
+            from robotmcp.core.library_manager import LibraryManager
+            
+            library_manager = LibraryManager()
+            
+            # Check if AppiumLibrary is available in the loaded libraries
+            appium_lib = None
+            
+            # First, try to get it from the library manager's loaded libraries
+            if hasattr(library_manager, '_libraries'):
+                for lib_name, lib_instance in library_manager._libraries.items():
+                    if lib_name == "AppiumLibrary" or "appium" in lib_name.lower():
+                        appium_lib = lib_instance
+                        break
+            
+            # If not found, try importing and getting instance directly
+            if not appium_lib:
+                try:
+                    import importlib
+                    appium_module = importlib.import_module('AppiumLibrary')
+                    # This might not work as libraries usually need proper initialization
+                    logger.debug("AppiumLibrary module imported but instance not available")
+                    return None
+                except ImportError:
+                    logger.debug("AppiumLibrary not installed")
+                    return None
+            
+            # Call get_source method directly on the library instance
+            if appium_lib and hasattr(appium_lib, 'get_source'):
+                source = appium_lib.get_source()
+                logger.debug(f"Got mobile source directly from AppiumLibrary: {len(source) if source else 0} chars")
+                return source
+            else:
+                logger.debug("AppiumLibrary instance found but get_source method not available")
+                return None
+                
+        except Exception as e:
+            logger.debug(f"Direct mobile source retrieval failed: {e}")
+            return None
